.copybtn {
  position: sticky;
  display: flex;
  top: 0;
  left: 100%;
  width: 1.7em;
  height: 1.7em;
  opacity: 0;
  transition: opacity 0.3s, border 0.3s, background-color 0.3s;
  user-select: none;
  padding: 0;
  border: none;
  outline: none;
  border-radius: 0.4em;
  border: #1b1f2426 1px solid;
  background-color: #f6f8fa;
  color: #57606a;
}
.codehilite pre {
  margin-top: -1.7em;
}
.copybtn svg {
  stroke: currentColor;
  width: 1.5em;
  height: 1.5em;
  padding: 0.1em;
}
.codehilite:hover .copybtn {
  opacity: 1;
}
