declare module "styles/icon.css" {
  export const panel_models_icon_ToggleIcon: string
  export const IconLabel: string
  export const IconRow: string
  const __css__: string
  export default __css__
}
