.error-wrapper {
  display: contents;
}
.error {
  padding: 0.75rem 1.25rem;
  border: 1px solid transparent;
  border-radius: 0.25rem;
  color: var(--danger-text-color);
  background-color: var(--danger-bg-subtle);
  border-color: var(--danger-border-subtle);
}
.error .msg {
  font-weight: bold;
}
.error pre.highlight {
  backdrop-filter: brightness(0.8);
  font-weight: 900;
}
