export const input_group: string
export const input: string
export const spin_wrapper: string
export const spin_btn_up: string
export const spin_btn_down: string
const __css__: string
export default __css__
