export const panel_models_markup_HTML = "bk-panel-models-markup-HTML"
export default `.tabulator-table{max-width:100%;}.tabulator-table .tabulator-row .row-content .bk-panel-models-markup-HTML{white-space:normal;}`
