/*!
 * FilePondPluginPdfPreview 1.0.2
 * Licensed under MIT, https://opensource.org/licenses/MIT/
 * Please visit undefined for details.
 */



.filepond--pdf-preview-wrapper {
  position: absolute;
  left: 0;
  top: 0;
  right: 0;
  height: 100%;
  margin: 0;
  border-radius: 0.45em;
  overflow: hidden;
  background: rgba(0, 0, 0, 0.01);
  pointer-events: auto;
}

.filepond--pdf-preview-wrapper:before {
  content: ' ';
  position: absolute;
  width: 100%;
  height: 2em;
  background: -moz-linear-gradient(top,  rgba(0,0,0,1) 0%, rgba(0,0,0,0) 100%); 
  /* FF3.6-15 */
  background: -webkit-linear-gradient(top,  rgba(0,0,0,1) 0%,rgba(0,0,0,0) 100%);
  /* Chrome10-25,Safari5.1-6 */
  background: linear-gradient(to bottom, black 0%, rgba(0, 0, 0, 0) 100%);
  /* W3C, IE10+, FF16+, Chrome26+, Opera12+, Safari7+ */
  filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#000000', endColorstr='#00000000',GradientType=0 );
  /* IE6-9 */
  z-index: 3;
}

.filepond--pdf-preview {
  position: relative;
  z-index: 1;
  display: block;
  width: 100%;
  height: 100%;
  -webkit-transform-origin: center center;
  transform-origin: center center;
  will-change: transform, opacity;
}

.filepond--pdf-preview #outerContainer #mainContainer div.toolbar {
  display: none !important; /* hide PDF viewer toolbar */
}

.filepond--pdf-preview #outerContainer #mainContainer #viewerContainer {
  top: 0 !important; /* move doc up into empty bar space */
}

