export default `.card-header{border-bottom:1px solid var(--panel-border-color, #e0e0e0);padding-block:15px;margin-block:5px;box-shadow:color-mix(in srgb, var(--panel-shadow-color) 30%, transparent) 0px 1px 2px
      0px,
    color-mix(in srgb, var(--panel-shadow-color) 15%, transparent) 0px 1px 3px
      1px;background-color:var(--panel-surface-color, #f1f1f1);}.card-header:not(:hover){box-shadow:color-mix(in srgb, var(--panel-shadow-color) 30%, transparent) 0px 1px 2px
      0px,
    color-mix(in srgb, var(--panel-shadow-color) 15%, transparent) 0px 1px 3px
      1px;}:host(.card-title){font-size:1.25em;padding-inline:5px;margin-left:0px;font-weight:500;}`
